import numpy as np
import laspy
from itertools import groupby

lazData_path = ["server/example_lidar_data/ot_N_000005_1.laz", "server/example_lidar_data/ot_N_000033_1.laz"]

# Info about data
with laspy.open(lazData_path[0]) as fh:
    # Print metadata properties
    print("File Version:", fh.header.version)
    print("Point Count:", fh.header.point_count)
    print("Scale Factors:", fh.header.scale)
    print("Offset:", fh.header.offset)

    print('Points from Header:', fh.header.point_count)
    las = fh.read()
    print(las)
    print('Points from data:', len(las.points))
    ground_pts = las.classification == 2
    bins, counts = np.unique(las.return_number[ground_pts], return_counts=True)
    print('Ground Point Return Number distribution:')
    for r, c in zip(bins, counts):
        print('    {}:{}'.format(r, c))

def inArea(position, areaRange):
    x, y, _ = position
    if (areaRange[0][0] < x < areaRange[1][0]) and (areaRange[0][1] > y > areaRange[1][1]):
        return True
    else:
        return False

# Calculation of height gotten from las files
def find_height(points):
    sorted_coords = sorted(points, key=lambda coord: (coord[0],coord[1]))

    groupCoords = [list(group) for key, group in groupby(sorted_coords, key=lambda coord: (coord[0], coord[1]))]

    height_differences = []
    groupss = []

    for groups in groupCoords:
        if len(groups) < 2 or groups[0] == groups[1]:
            continue

        min_height = min(coords[2] for coords in groups)
        max_height = max(coords[2] for coords in groups)

        difference = max_height - min_height
        height_differences.append(difference)
        groupss.append(groups)

    return height_differences

# areas
def height_in_area(area):
    # Limit the data to specific areas
    
    # this is only temporary data for coordinates in arctic
    # areas = [
    #     [(-3671212, 7898422), (-3200175, 4699978)],
    #     [(60.815356, 10.672022), (60.774878, 10.768867)],
    # ]

    # NB: is only for the test data should be removed after
    areazone = [(((area[0]) * ((-3671212/60.815356) - (-3200175/60.774878))) - 3200175, (area[1] * ((7898422/10.768867) - (4699978/10.672022))) + 4699978),
                (((area[0] - 100) * ((-3671212/60.815356) - (-3200175/60.774878))) - 3200175, ((area[1] - 1) * ((7898422/10.768867) - (4699978/10.672022))) + 4699978)]

    # Refactor data format
    iceOver = laspy.read(lazData_path[0])
    iceUnder = laspy.read(lazData_path[1])

    ice_points = list(zip(iceOver.X,iceOver.Y,iceOver.Z)) + list(zip(iceUnder.X,iceUnder.Y,iceUnder.Z))

    area_heights = []

    ice_points = list(filter(lambda position: inArea(position, areazone), ice_points))

    area_heights = find_height(ice_points)

    return area_heights
